# document_manager.py
import requests
from docxtpl import DocxTemplate
from docx import Document
import time
import re

class AIDocumentGenerator:
    """
    AIDocumentGenerator Class

    This class provides functionality for managing document generation using the Gemini Google API,
    filling placeholders in Microsoft Word documents, and extracting values from Word documents.

    Attributes:
        api_key (str): The API key for accessing the Gemini Google API.
        gemini_url (str): The URL for the Gemini Google API.

    Methods:
        extract_content(prompt: str) -> str:
            Fetch content from the Gemini Google API based on the provided prompt.

        generate_and_save_document(document_path: str, save_path: str, processing_dict: dict) -> None:
            Generate and save a Microsoft Word document by filling placeholders with data.

        extract_values_from_docx(docx_path: str) -> list:
            Extract values enclosed in double curly braces ({{ ... }}) from a Microsoft Word document.

        get_user_input_for_placeholders(placeholders) -> dict:
            Get user input for a list of placeholders and create a dictionary with trimmed keys.

        display_progress_bar(total_steps: int, desc: str = "Generating Document") -> None:
            Display a progress bar in the terminal for a specified number of steps.

        create_ai_document(document_path, save_path) -> None:
            Main method for creating an AI-generated document. Takes user input for placeholders,
            generates content using the Gemini Google API, and saves the document.

    Usage Example:
        dm = AIDocumentGenerator()
        dm.create_ai_document("path/to/template.docx", "path/to/output.docx")
    """
    def __init__(self):
        self.api_key = "AIzaSyCMS0L9Nd3VeQyn4BsNBTcz22fqYtIpfFc"
        self.gemini_url = 'https://generativelanguage.googleapis.com/v1beta/models/gemini-pro:generateContent?key=' + self.api_key

    def extract_content(self, prompt: str) -> str:
        """
        Fetch content from the Gemini Google API based on the provided prompt.

        Parameters:
            prompt (str): The prompt for generating content using the Gemini Google API.

        Returns:
            str: The generated content from the API response.

        Raises:
            requests.exceptions.RequestException: If there is an issue with the HTTP request.
            Exception: For unexpected errors during API request or response processing.

        Example:
            >>> result = extract_content("Write a program in Python to know if a number is prime or not.")
            >>> print(result)
            "Here is the generated content based on the provided prompt."
        """
        data = {
            "contents": [
                {
                    "parts": [
                        {
                            "text": f"{prompt}"
                        }
                    ]
                }
            ]
        }
        headers = {
            'Content-Type': 'application/json'
        }
        try:
            response = requests.post(self.gemini_url, json=data, headers=headers, verify=False)
            result = response.json()
            answer = result["candidates"][0]["content"]
            return answer["parts"][0]["text"]
        except requests.exceptions.RequestException as e:
            print(f"Error: {e}")
        except Exception as e:
            print(f"Unexpected error: {e}")

    def generate_and_save_document(self, document_path: str, save_path: str, processing_dict: dict) -> None:
        """
        Generate and save a Microsoft Word document by filling placeholders with data.

        Parameters:
            document_path (str): The path to the template Word document.
            save_path (str): The path to save the generated Word document.
            processing_dict (dict): A dictionary containing key-value pairs for filling placeholders in the document.

        Returns:
            None: The function saves the generated document to the specified path.

        Example:
            >>> generate_and_save_document('template/document_template.docx', 'output/generated_document.docx', {'name': 'John Doe'})
        """
        try:
            doc = DocxTemplate(str(document_path))
            data = processing_dict
            doc.render(data)
            self.display_progress_bar(100)
            doc.save(save_path)
        except PermissionError as e:
            print(f"PermissionError: {e}")
            print("Unable to save the document. Please check file permissions.")
            print("Please close the file!")
        except Exception as e:
            print(f"An unexpected error occurred: {e}")

        values = self.extract_values_from_docx(document_path)
        """
        Extract values enclosed in double curly braces ({{ ... }}) from a Microsoft Word document.

        Parameters:
            docx_path (str): The path to the Word document.

        Returns:
            list: A list of extracted values found in the specified pattern.

        Example:
            >>> values = extract_values_from_docx('path/to/your/document.docx')
            >>> print(values)
            ['value1', 'value2', 'value3']
        """
        print("Extracted Values:")
        for value in values:
            print(type(value), value)

    def extract_values_from_docx(self, docx_path: str) -> list:
        """
        Extract values enclosed in double curly braces ({{ ... }}) from a Microsoft Word document.

        Parameters:
            docx_path (str): The path to the Word document.

        Returns:
            list: A list of extracted values found in the specified pattern.

        Example:
            >>> values = extract_values_from_docx('path/to/your/document.docx')
            >>> print(values)
            ['value1', 'value2', 'value3']
        """
        doc = Document(docx_path)
        pattern = r'\{\{([^}]+)\}\}'
        extracted_values = []
        for paragraph in doc.paragraphs:
            matches = re.findall(pattern, paragraph.text)
            extracted_values.extend(matches)
        return extracted_values

    def get_user_input_for_placeholders(self, placeholders):
        """
        Get user input for a list of placeholders and create a dictionary with trimmed keys.

        Parameters:
            placeholders (list): A list of placeholders for which user input is required.

        Returns:
            dict: A dictionary with trimmed keys where the user input for each placeholder is stored.

        Example:
            >>> placeholders = [' aiData ', 'another_placeholder', 'yet_another']
            >>> user_input_dict = get_user_input_for_placeholders(placeholders)
            Enter value for ' aiData ': User's Input 1
            Enter value for 'another_placeholder': User's Input 2
            Enter value for 'yet_another': User's Input 3
            >>> print(user_input_dict)
            {'aiData': 'User's Input 1', 'another_placeholder': 'User's Input 2', 'yet_another': 'User's Input 3'}
        """
        user_input_dict = {}
        for placeholder in placeholders:
            user_input = input(f"Enter prompt for '{placeholder}': ")
            trimmed_placeholder = placeholder.strip()
            user_input_dict[trimmed_placeholder] = user_input
        return user_input_dict

    def display_progress_bar(self, total_steps: int, desc: str = "Generating Document") -> None:
        print(desc + ": ", end="")
        for _ in range(total_steps):
            time.sleep(0.1)
            print("█", end="", flush=True)
        print(" 100%")

    def create_ai_document(self, document_path, save_path):
        values = self.extract_values_from_docx(document_path)
        user_input_dict = self.get_user_input_for_placeholders(values)

        my_placeholder_dict = {
            key: self.extract_content(user_input_dict[key])
            for key in user_input_dict
        }

        self.generate_and_save_document(document_path, save_path, my_placeholder_dict)
        print(f"Document Successfully 🎉 Generated and Saved to this Path 👉 {save_path}")

# if __name__ == "__main__":
#     print("""
#       /$$$$$$      /$$$$$$           /$$$$$$$                                                                    /$$    
#      /$$__  $$    |_  $$_/          | $$__  $$                                                                  | $$    
#     | $$  \\ $$      | $$            | $$  \\ $$  /$$$$$$   /$$$$$$$ /$$   /$$ /$$$$$$/$$$$   /$$$$$$  /$$$$$$$  /$$$$$$  
#     | $$$$$$$$      | $$            | $$  | $$ /$$__  $$ /$$_____/| $$  | $$| $$_  $$_  $$ /$$__  $$| $$__  $$|_  $$_/  
#     | $$__  $$      | $$            | $$  | $$| $$  \\ $$| $$      | $$  | $$| $$ \\ $$ \\ $$| $$$$$$$$| $$  \\ $$  | $$    
#     | $$  | $$      | $$            | $$  | $$| $$  | $$| $$      | $$  | $$| $$ | $$ | $$| $$_____/| $$  | $$  | $$ /$$
#     | $$  | $$ /$$ /$$$$$$ /$$      | $$$$$$$/|  $$$$$$/|  $$$$$$$|  $$$$$$/| $$ | $$ | $$|  $$$$$$$| $$  | $$  |  $$$$/
#     |__/  |__/|__/|______/|__/      |_______/  \\______/  \\_______/ \\______/ |__/ |__/ |__/ \\_______/|__/  |__/   \\___/  

#      /$$$$$$                                                     /$$                                                   
#      /$$__  $$                                                   | $$                                                   
#     | $$  \\__/  /$$$$$$  /$$$$$$$   /$$$$$$   /$$$$$$  /$$$$$$  /$$$$$$    /$$$$$$   /$$$$$$                            
#     | $$ /$$$$ /$$__  $$| $$__  $$ /$$__  $$ /$$__  $$|____  $$|_  $$_/   /$$__  $$ /$$__  $$                           
#     | $$|_  $$| $$$$$$$$| $$  \\ $$| $$$$$$$$| $$  \\__/ /$$$$$$$  | $$    | $$  \\ $$| $$  \\__/                           
#     | $$  \\ $$| $$_____/| $$  | $$| $$_____/| $$      /$$__  $$  | $$ /$$| $$  | $$| $$                                 
#     |  $$$$$$/|  $$$$$$$| $$  | $$|  $$$$$$$| $$     |  $$$$$$$  |  $$$$/|  $$$$$$/| $$                                 
#     \\______/  \\_______/|__/  |__/ \\_______/|__/      \\_______/   \\___/   \\______/ |__/                   
#     """)
#     dm = AIDocumentGenerator()
#     dp = input("Give the whole document path?\n")
#     sp = input("Give the whole path to save the document!\n")
#     dm.create_ai_document(dp, sp)
